# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

import datetime
from typing import List, Optional

from .side import Side
from ..._models import BaseModel

__all__ = ["SignalGetEntrySignalsResponse", "Signal"]


class Signal(BaseModel):
    date: datetime.date
    """Date of the signal"""

    index: int
    """Index in the dataframe where signal occurred"""

    price: float
    """Price at signal time"""

    side: Side
    """Trading direction"""

    ticker: str
    """Ticker symbol"""

    time: str
    """Time of the signal"""

    condition: Optional[str] = None
    """Condition that triggered the signal"""

    noise_spread: Optional[float] = None
    """Noise spread factor"""


class SignalGetEntrySignalsResponse(BaseModel):
    signals: List[Signal]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .._models import BaseModel

__all__ = ["StrategyGetLeverageResponse"]


class StrategyGetLeverageResponse(BaseModel):
    leverage: float



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List

from .._models import BaseModel

__all__ = ["StrategyGetTickersResponse"]


class StrategyGetTickersResponse(BaseModel):
    tickers: List[str]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union, Optional
from datetime import date
from typing_extensions import Required, Annotated, TypedDict

from ..._utils import PropertyInfo
from ..strategy_config_param import StrategyConfigParam

__all__ = ["SignalGetEntrySignalsParams"]


class SignalGetEntrySignalsParams(TypedDict, total=False):
    strategy: Required[StrategyConfigParam]
    """Strategy configuration for instantiating a strategy"""

    effective_date: Annotated[Union[str, date, None], PropertyInfo(format="iso8601")]
    """Filter by date"""

    effective_time: Optional[str]
    """Filter by time"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

from .strategy_config_param import StrategyConfigParam

__all__ = ["StrategyGetTradingDaysParams"]


class StrategyGetTradingDaysParams(TypedDict, total=False):
    strategy: Required[StrategyConfigParam]
    """Strategy configuration for instantiating a strategy"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from tests.utils import assert_matches_type
from untapd_strategy import UntapdStrategy, AsyncUntapdStrategy
from untapd_strategy._utils import parse_date
from untapd_strategy.types.strategy import (
    SignalGetExitSignalsResponse,
    SignalGetEntrySignalsResponse,
)

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestSignals:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_get_entry_signals(self, client: UntapdStrategy) -> None:
        signal = client.strategy.signals.get_entry_signals(
            strategy={"strategy_name": "poseidon"},
        )
        assert_matches_type(SignalGetEntrySignalsResponse, signal, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_get_entry_signals_with_all_params(self, client: UntapdStrategy) -> None:
        signal = client.strategy.signals.get_entry_signals(
            strategy={
                "strategy_name": "poseidon",
                "config": {
                    "end_date": "bar",
                    "silent": "bar",
                    "start_date": "bar",
                    "tickers": "bar",
                },
            },
            effective_date=parse_date("2024-01-15"),
            effective_time="18:11:19.117Z",
        )
        assert_matches_type(SignalGetEntrySignalsResponse, signal, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_get_entry_signals(self, client: UntapdStrategy) -> None:
        response = client.strategy.signals.with_raw_response.get_entry_signals(
            strategy={"strategy_name": "poseidon"},
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        signal = response.parse()
        assert_matches_type(SignalGetEntrySignalsResponse, signal, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_get_entry_signals(self, client: UntapdStrategy) -> None:
        with client.strategy.signals.with_streaming_response.get_entry_signals(
            strategy={"strategy_name": "poseidon"},
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            signal = response.parse()
            assert_matches_type(SignalGetEntrySignalsResponse, signal, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_get_exit_signals(self, client: UntapdStrategy) -> None:
        signal = client.strategy.signals.get_exit_signals(
            portfolio=[
                {
                    "qty": 100,
                    "side": "LONG",
                    "ticker": "SPY",
                }
            ],
            strategy={"strategy_name": "poseidon"},
        )
        assert_matches_type(SignalGetExitSignalsResponse, signal, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_get_exit_signals_with_all_params(self, client: UntapdStrategy) -> None:
        signal = client.strategy.signals.get_exit_signals(
            portfolio=[
                {
                    "qty": 100,
                    "side": "LONG",
                    "ticker": "SPY",
                }
            ],
            strategy={
                "strategy_name": "poseidon",
                "config": {
                    "end_date": "bar",
                    "silent": "bar",
                    "start_date": "bar",
                    "tickers": "bar",
                },
            },
            effective_date=parse_date("2024-01-15"),
            effective_time="15:30:00",
        )
        assert_matches_type(SignalGetExitSignalsResponse, signal, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_get_exit_signals(self, client: UntapdStrategy) -> None:
        response = client.strategy.signals.with_raw_response.get_exit_signals(
            portfolio=[
                {
                    "qty": 100,
                    "side": "LONG",
                    "ticker": "SPY",
                }
            ],
            strategy={"strategy_name": "poseidon"},
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        signal = response.parse()
        assert_matches_type(SignalGetExitSignalsResponse, signal, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_get_exit_signals(self, client: UntapdStrategy) -> None:
        with client.strategy.signals.with_streaming_response.get_exit_signals(
            portfolio=[
                {
                    "qty": 100,
                    "side": "LONG",
                    "ticker": "SPY",
                }
            ],
            strategy={"strategy_name": "poseidon"},
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            signal = response.parse()
            assert_matches_type(SignalGetExitSignalsResponse, signal, path=["response"])

        assert cast(Any, response.is_closed) is True


class TestAsyncSignals:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_get_entry_signals(self, async_client: AsyncUntapdStrategy) -> None:
        signal = await async_client.strategy.signals.get_entry_signals(
            strategy={"strategy_name": "poseidon"},
        )
        assert_matches_type(SignalGetEntrySignalsResponse, signal, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_get_entry_signals_with_all_params(self, async_client: AsyncUntapdStrategy) -> None:
        signal = await async_client.strategy.signals.get_entry_signals(
            strategy={
                "strategy_name": "poseidon",
                "config": {
                    "end_date": "bar",
                    "silent": "bar",
                    "start_date": "bar",
                    "tickers": "bar",
                },
            },
            effective_date=parse_date("2024-01-15"),
            effective_time="18:11:19.117Z",
        )
        assert_matches_type(SignalGetEntrySignalsResponse, signal, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_get_entry_signals(self, async_client: AsyncUntapdStrategy) -> None:
        response = await async_client.strategy.signals.with_raw_response.get_entry_signals(
            strategy={"strategy_name": "poseidon"},
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        signal = await response.parse()
        assert_matches_type(SignalGetEntrySignalsResponse, signal, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_get_entry_signals(self, async_client: AsyncUntapdStrategy) -> None:
        async with async_client.strategy.signals.with_streaming_response.get_entry_signals(
            strategy={"strategy_name": "poseidon"},
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            signal = await response.parse()
            assert_matches_type(SignalGetEntrySignalsResponse, signal, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_get_exit_signals(self, async_client: AsyncUntapdStrategy) -> None:
        signal = await async_client.strategy.signals.get_exit_signals(
            portfolio=[
                {
                    "qty": 100,
                    "side": "LONG",
                    "ticker": "SPY",
                }
            ],
            strategy={"strategy_name": "poseidon"},
        )
        assert_matches_type(SignalGetExitSignalsResponse, signal, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_get_exit_signals_with_all_params(self, async_client: AsyncUntapdStrategy) -> None:
        signal = await async_client.strategy.signals.get_exit_signals(
            portfolio=[
                {
                    "qty": 100,
                    "side": "LONG",
                    "ticker": "SPY",
                }
            ],
            strategy={
                "strategy_name": "poseidon",
                "config": {
                    "end_date": "bar",
                    "silent": "bar",
                    "start_date": "bar",
                    "tickers": "bar",
                },
            },
            effective_date=parse_date("2024-01-15"),
            effective_time="15:30:00",
        )
        assert_matches_type(SignalGetExitSignalsResponse, signal, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_get_exit_signals(self, async_client: AsyncUntapdStrategy) -> None:
        response = await async_client.strategy.signals.with_raw_response.get_exit_signals(
            portfolio=[
                {
                    "qty": 100,
                    "side": "LONG",
                    "ticker": "SPY",
                }
            ],
            strategy={"strategy_name": "poseidon"},
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        signal = await response.parse()
        assert_matches_type(SignalGetExitSignalsResponse, signal, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_get_exit_signals(self, async_client: AsyncUntapdStrategy) -> None:
        async with async_client.strategy.signals.with_streaming_response.get_exit_signals(
            portfolio=[
                {
                    "qty": 100,
                    "side": "LONG",
                    "ticker": "SPY",
                }
            ],
            strategy={"strategy_name": "poseidon"},
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            signal = await response.parse()
            assert_matches_type(SignalGetExitSignalsResponse, signal, path=["response"])

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from datetime import date
from typing_extensions import Required, Annotated, TypedDict

from .._utils import PropertyInfo
from .strategy_config_param import StrategyConfigParam

__all__ = ["StrategyGetDataParams"]


class StrategyGetDataParams(TypedDict, total=False):
    strategy: Required[StrategyConfigParam]
    """Strategy configuration for instantiating a strategy"""

    ticker: Required[str]
    """Ticker symbol"""

    trading_date: Required[Annotated[Union[str, date], PropertyInfo(format="iso8601")]]
    """Trading date in YYYY-MM-DD format"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .._models import BaseModel

__all__ = ["StrategyGetNameResponse"]


class StrategyGetNameResponse(BaseModel):
    name: str



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, List

from .._models import BaseModel

__all__ = ["StrategyGetDataResponse"]


class StrategyGetDataResponse(BaseModel):
    data: List[Dict[str, object]]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .strategy_config_param import StrategyConfigParam as StrategyConfigParam
from .strategy_get_data_params import StrategyGetDataParams as StrategyGetDataParams
from .strategy_get_name_params import StrategyGetNameParams as StrategyGetNameParams
from .strategy_get_data_response import StrategyGetDataResponse as StrategyGetDataResponse
from .strategy_get_name_response import StrategyGetNameResponse as StrategyGetNameResponse
from .strategy_get_tickers_params import StrategyGetTickersParams as StrategyGetTickersParams
from .strategy_get_leverage_params import StrategyGetLeverageParams as StrategyGetLeverageParams
from .strategy_get_tickers_response import StrategyGetTickersResponse as StrategyGetTickersResponse
from .strategy_get_leverage_response import StrategyGetLeverageResponse as StrategyGetLeverageResponse
from .strategy_get_trading_days_params import StrategyGetTradingDaysParams as StrategyGetTradingDaysParams
from .strategy_get_trading_days_response import StrategyGetTradingDaysResponse as StrategyGetTradingDaysResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .health import (
    HealthResource,
    AsyncHealthResource,
    HealthResourceWithRawResponse,
    AsyncHealthResourceWithRawResponse,
    HealthResourceWithStreamingResponse,
    AsyncHealthResourceWithStreamingResponse,
)
from .strategy import (
    StrategyResource,
    AsyncStrategyResource,
    StrategyResourceWithRawResponse,
    AsyncStrategyResourceWithRawResponse,
    StrategyResourceWithStreamingResponse,
    AsyncStrategyResourceWithStreamingResponse,
)

__all__ = [
    "HealthResource",
    "AsyncHealthResource",
    "HealthResourceWithRawResponse",
    "AsyncHealthResourceWithRawResponse",
    "HealthResourceWithStreamingResponse",
    "AsyncHealthResourceWithStreamingResponse",
    "StrategyResource",
    "AsyncStrategyResource",
    "StrategyResourceWithRawResponse",
    "AsyncStrategyResourceWithRawResponse",
    "StrategyResourceWithStreamingResponse",
    "AsyncStrategyResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict
from typing_extensions import Required, TypedDict

__all__ = ["StrategyConfigParam"]


class StrategyConfigParam(TypedDict, total=False):
    strategy_name: Required[str]
    """Name of the strategy class (e.g., 'PoseidonStrategy' or 'poseidon')"""

    config: Dict[str, object]
    """Strategy initialization parameters"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .side import Side as Side
from .signal_get_exit_signals_params import SignalGetExitSignalsParams as SignalGetExitSignalsParams
from .signal_get_entry_signals_params import SignalGetEntrySignalsParams as SignalGetEntrySignalsParams
from .signal_get_exit_signals_response import SignalGetExitSignalsResponse as SignalGetExitSignalsResponse
from .signal_get_entry_signals_response import SignalGetEntrySignalsResponse as SignalGetEntrySignalsResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .signals import (
    SignalsResource,
    AsyncSignalsResource,
    SignalsResourceWithRawResponse,
    AsyncSignalsResourceWithRawResponse,
    SignalsResourceWithStreamingResponse,
    AsyncSignalsResourceWithStreamingResponse,
)
from .strategy import (
    StrategyResource,
    AsyncStrategyResource,
    StrategyResourceWithRawResponse,
    AsyncStrategyResourceWithRawResponse,
    StrategyResourceWithStreamingResponse,
    AsyncStrategyResourceWithStreamingResponse,
)

__all__ = [
    "SignalsResource",
    "AsyncSignalsResource",
    "SignalsResourceWithRawResponse",
    "AsyncSignalsResourceWithRawResponse",
    "SignalsResourceWithStreamingResponse",
    "AsyncSignalsResourceWithStreamingResponse",
    "StrategyResource",
    "AsyncStrategyResource",
    "StrategyResourceWithRawResponse",
    "AsyncStrategyResourceWithRawResponse",
    "StrategyResourceWithStreamingResponse",
    "AsyncStrategyResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from datetime import date

import httpx

from ...types import (
    strategy_get_data_params,
    strategy_get_name_params,
    strategy_get_tickers_params,
    strategy_get_leverage_params,
    strategy_get_trading_days_params,
)
from .signals import (
    SignalsResource,
    AsyncSignalsResource,
    SignalsResourceWithRawResponse,
    AsyncSignalsResourceWithRawResponse,
    SignalsResourceWithStreamingResponse,
    AsyncSignalsResourceWithStreamingResponse,
)
from ..._types import Body, Query, Headers, NotGiven, not_given
from ..._utils import maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._base_client import make_request_options
from ...types.strategy_config_param import StrategyConfigParam
from ...types.strategy_get_data_response import StrategyGetDataResponse
from ...types.strategy_get_name_response import StrategyGetNameResponse
from ...types.strategy_get_tickers_response import StrategyGetTickersResponse
from ...types.strategy_get_leverage_response import StrategyGetLeverageResponse
from ...types.strategy_get_trading_days_response import StrategyGetTradingDaysResponse

__all__ = ["StrategyResource", "AsyncStrategyResource"]


class StrategyResource(SyncAPIResource):
    @cached_property
    def signals(self) -> SignalsResource:
        return SignalsResource(self._client)

    @cached_property
    def with_raw_response(self) -> StrategyResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/untapd-labs/strategy-sdk#accessing-raw-response-data-eg-headers
        """
        return StrategyResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> StrategyResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/untapd-labs/strategy-sdk#with_streaming_response
        """
        return StrategyResourceWithStreamingResponse(self)

    def get_data(
        self,
        *,
        strategy: StrategyConfigParam,
        ticker: str,
        trading_date: Union[str, date],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> StrategyGetDataResponse:
        """
        Get OHLCV and strategy-specific data for a given trading date and ticker

        Requires authentication via X-API-Key header.

        Args:
          strategy: Strategy configuration for instantiating a strategy

          ticker: Ticker symbol

          trading_date: Trading date in YYYY-MM-DD format

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/v1/strategy/data",
            body=maybe_transform(
                {
                    "strategy": strategy,
                    "ticker": ticker,
                    "trading_date": trading_date,
                },
                strategy_get_data_params.StrategyGetDataParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=StrategyGetDataResponse,
        )

    def get_leverage(
        self,
        *,
        max_leverage: float,
        strategy: StrategyConfigParam,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> StrategyGetLeverageResponse:
        """
        Calculate the leverage to use for trades based on strategy logic

        Requires authentication via X-API-Key header.

        Args:
          max_leverage: Maximum allowed leverage

          strategy: Strategy configuration for instantiating a strategy

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/v1/strategy/leverage",
            body=maybe_transform(
                {
                    "max_leverage": max_leverage,
                    "strategy": strategy,
                },
                strategy_get_leverage_params.StrategyGetLeverageParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=StrategyGetLeverageResponse,
        )

    def get_name(
        self,
        *,
        strategy: StrategyConfigParam,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> StrategyGetNameResponse:
        """
        Get the name of the strategy

        Requires authentication via X-API-Key header.

        Args:
          strategy: Strategy configuration for instantiating a strategy

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/v1/strategy/name",
            body=maybe_transform({"strategy": strategy}, strategy_get_name_params.StrategyGetNameParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=StrategyGetNameResponse,
        )

    def get_tickers(
        self,
        *,
        strategy: StrategyConfigParam,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> StrategyGetTickersResponse:
        """
        Get the list of tickers the strategy is designed for

        Requires authentication via X-API-Key header.

        Args:
          strategy: Strategy configuration for instantiating a strategy

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/v1/strategy/tickers",
            body=maybe_transform({"strategy": strategy}, strategy_get_tickers_params.StrategyGetTickersParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=StrategyGetTickersResponse,
        )

    def get_trading_days(
        self,
        *,
        strategy: StrategyConfigParam,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> StrategyGetTradingDaysResponse:
        """
        Get the list of unique trading days available in the strategy

        Requires authentication via X-API-Key header.

        Args:
          strategy: Strategy configuration for instantiating a strategy

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/v1/strategy/trading-days",
            body=maybe_transform({"strategy": strategy}, strategy_get_trading_days_params.StrategyGetTradingDaysParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=StrategyGetTradingDaysResponse,
        )


class AsyncStrategyResource(AsyncAPIResource):
    @cached_property
    def signals(self) -> AsyncSignalsResource:
        return AsyncSignalsResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncStrategyResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/untapd-labs/strategy-sdk#accessing-raw-response-data-eg-headers
        """
        return AsyncStrategyResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncStrategyResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/untapd-labs/strategy-sdk#with_streaming_response
        """
        return AsyncStrategyResourceWithStreamingResponse(self)

    async def get_data(
        self,
        *,
        strategy: StrategyConfigParam,
        ticker: str,
        trading_date: Union[str, date],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> StrategyGetDataResponse:
        """
        Get OHLCV and strategy-specific data for a given trading date and ticker

        Requires authentication via X-API-Key header.

        Args:
          strategy: Strategy configuration for instantiating a strategy

          ticker: Ticker symbol

          trading_date: Trading date in YYYY-MM-DD format

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/v1/strategy/data",
            body=await async_maybe_transform(
                {
                    "strategy": strategy,
                    "ticker": ticker,
                    "trading_date": trading_date,
                },
                strategy_get_data_params.StrategyGetDataParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=StrategyGetDataResponse,
        )

    async def get_leverage(
        self,
        *,
        max_leverage: float,
        strategy: StrategyConfigParam,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> StrategyGetLeverageResponse:
        """
        Calculate the leverage to use for trades based on strategy logic

        Requires authentication via X-API-Key header.

        Args:
          max_leverage: Maximum allowed leverage

          strategy: Strategy configuration for instantiating a strategy

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/v1/strategy/leverage",
            body=await async_maybe_transform(
                {
                    "max_leverage": max_leverage,
                    "strategy": strategy,
                },
                strategy_get_leverage_params.StrategyGetLeverageParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=StrategyGetLeverageResponse,
        )

    async def get_name(
        self,
        *,
        strategy: StrategyConfigParam,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> StrategyGetNameResponse:
        """
        Get the name of the strategy

        Requires authentication via X-API-Key header.

        Args:
          strategy: Strategy configuration for instantiating a strategy

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/v1/strategy/name",
            body=await async_maybe_transform({"strategy": strategy}, strategy_get_name_params.StrategyGetNameParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=StrategyGetNameResponse,
        )

    async def get_tickers(
        self,
        *,
        strategy: StrategyConfigParam,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> StrategyGetTickersResponse:
        """
        Get the list of tickers the strategy is designed for

        Requires authentication via X-API-Key header.

        Args:
          strategy: Strategy configuration for instantiating a strategy

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/v1/strategy/tickers",
            body=await async_maybe_transform(
                {"strategy": strategy}, strategy_get_tickers_params.StrategyGetTickersParams
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=StrategyGetTickersResponse,
        )

    async def get_trading_days(
        self,
        *,
        strategy: StrategyConfigParam,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> StrategyGetTradingDaysResponse:
        """
        Get the list of unique trading days available in the strategy

        Requires authentication via X-API-Key header.

        Args:
          strategy: Strategy configuration for instantiating a strategy

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/v1/strategy/trading-days",
            body=await async_maybe_transform(
                {"strategy": strategy}, strategy_get_trading_days_params.StrategyGetTradingDaysParams
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=StrategyGetTradingDaysResponse,
        )


class StrategyResourceWithRawResponse:
    def __init__(self, strategy: StrategyResource) -> None:
        self._strategy = strategy

        self.get_data = to_raw_response_wrapper(
            strategy.get_data,
        )
        self.get_leverage = to_raw_response_wrapper(
            strategy.get_leverage,
        )
        self.get_name = to_raw_response_wrapper(
            strategy.get_name,
        )
        self.get_tickers = to_raw_response_wrapper(
            strategy.get_tickers,
        )
        self.get_trading_days = to_raw_response_wrapper(
            strategy.get_trading_days,
        )

    @cached_property
    def signals(self) -> SignalsResourceWithRawResponse:
        return SignalsResourceWithRawResponse(self._strategy.signals)


class AsyncStrategyResourceWithRawResponse:
    def __init__(self, strategy: AsyncStrategyResource) -> None:
        self._strategy = strategy

        self.get_data = async_to_raw_response_wrapper(
            strategy.get_data,
        )
        self.get_leverage = async_to_raw_response_wrapper(
            strategy.get_leverage,
        )
        self.get_name = async_to_raw_response_wrapper(
            strategy.get_name,
        )
        self.get_tickers = async_to_raw_response_wrapper(
            strategy.get_tickers,
        )
        self.get_trading_days = async_to_raw_response_wrapper(
            strategy.get_trading_days,
        )

    @cached_property
    def signals(self) -> AsyncSignalsResourceWithRawResponse:
        return AsyncSignalsResourceWithRawResponse(self._strategy.signals)


class StrategyResourceWithStreamingResponse:
    def __init__(self, strategy: StrategyResource) -> None:
        self._strategy = strategy

        self.get_data = to_streamed_response_wrapper(
            strategy.get_data,
        )
        self.get_leverage = to_streamed_response_wrapper(
            strategy.get_leverage,
        )
        self.get_name = to_streamed_response_wrapper(
            strategy.get_name,
        )
        self.get_tickers = to_streamed_response_wrapper(
            strategy.get_tickers,
        )
        self.get_trading_days = to_streamed_response_wrapper(
            strategy.get_trading_days,
        )

    @cached_property
    def signals(self) -> SignalsResourceWithStreamingResponse:
        return SignalsResourceWithStreamingResponse(self._strategy.signals)


class AsyncStrategyResourceWithStreamingResponse:
    def __init__(self, strategy: AsyncStrategyResource) -> None:
        self._strategy = strategy

        self.get_data = async_to_streamed_response_wrapper(
            strategy.get_data,
        )
        self.get_leverage = async_to_streamed_response_wrapper(
            strategy.get_leverage,
        )
        self.get_name = async_to_streamed_response_wrapper(
            strategy.get_name,
        )
        self.get_tickers = async_to_streamed_response_wrapper(
            strategy.get_tickers,
        )
        self.get_trading_days = async_to_streamed_response_wrapper(
            strategy.get_trading_days,
        )

    @cached_property
    def signals(self) -> AsyncSignalsResourceWithStreamingResponse:
        return AsyncSignalsResourceWithStreamingResponse(self._strategy.signals)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union, Iterable, Optional
from datetime import date
from typing_extensions import Required, Annotated, TypedDict

from .side import Side
from ..._utils import PropertyInfo
from ..strategy_config_param import StrategyConfigParam

__all__ = ["SignalGetExitSignalsParams", "Portfolio"]


class SignalGetExitSignalsParams(TypedDict, total=False):
    portfolio: Required[Iterable[Portfolio]]

    strategy: Required[StrategyConfigParam]
    """Strategy configuration for instantiating a strategy"""

    effective_date: Annotated[Union[str, date, None], PropertyInfo(format="iso8601")]
    """Filter by date"""

    effective_time: Optional[str]
    """Filter by time"""


class Portfolio(TypedDict, total=False):
    qty: Required[float]
    """Quantity of shares"""

    side: Required[Side]
    """Position direction"""

    ticker: Required[str]
    """Ticker symbol"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union, Iterable, Optional
from datetime import date

import httpx

from ..._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ..._utils import maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._base_client import make_request_options
from ...types.strategy import signal_get_exit_signals_params, signal_get_entry_signals_params
from ...types.strategy_config_param import StrategyConfigParam
from ...types.strategy.signal_get_exit_signals_response import SignalGetExitSignalsResponse
from ...types.strategy.signal_get_entry_signals_response import SignalGetEntrySignalsResponse

__all__ = ["SignalsResource", "AsyncSignalsResource"]


class SignalsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> SignalsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/untapd-labs/strategy-sdk#accessing-raw-response-data-eg-headers
        """
        return SignalsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> SignalsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/untapd-labs/strategy-sdk#with_streaming_response
        """
        return SignalsResourceWithStreamingResponse(self)

    def get_entry_signals(
        self,
        *,
        strategy: StrategyConfigParam,
        effective_date: Union[str, date, None] | Omit = omit,
        effective_time: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SignalGetEntrySignalsResponse:
        """
        Get entry signals for the strategy

        Requires authentication via X-API-Key header.

        Args:
          strategy: Strategy configuration for instantiating a strategy

          effective_date: Filter by date

          effective_time: Filter by time

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/v1/strategy/signals/entry",
            body=maybe_transform(
                {
                    "strategy": strategy,
                    "effective_date": effective_date,
                    "effective_time": effective_time,
                },
                signal_get_entry_signals_params.SignalGetEntrySignalsParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SignalGetEntrySignalsResponse,
        )

    def get_exit_signals(
        self,
        *,
        portfolio: Iterable[signal_get_exit_signals_params.Portfolio],
        strategy: StrategyConfigParam,
        effective_date: Union[str, date, None] | Omit = omit,
        effective_time: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SignalGetExitSignalsResponse:
        """
        Get exit signals for the given portfolio positions

        Requires authentication via X-API-Key header.

        Args:
          strategy: Strategy configuration for instantiating a strategy

          effective_date: Filter by date

          effective_time: Filter by time

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/v1/strategy/signals/exit",
            body=maybe_transform(
                {
                    "portfolio": portfolio,
                    "strategy": strategy,
                    "effective_date": effective_date,
                    "effective_time": effective_time,
                },
                signal_get_exit_signals_params.SignalGetExitSignalsParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SignalGetExitSignalsResponse,
        )


class AsyncSignalsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncSignalsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/untapd-labs/strategy-sdk#accessing-raw-response-data-eg-headers
        """
        return AsyncSignalsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncSignalsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/untapd-labs/strategy-sdk#with_streaming_response
        """
        return AsyncSignalsResourceWithStreamingResponse(self)

    async def get_entry_signals(
        self,
        *,
        strategy: StrategyConfigParam,
        effective_date: Union[str, date, None] | Omit = omit,
        effective_time: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SignalGetEntrySignalsResponse:
        """
        Get entry signals for the strategy

        Requires authentication via X-API-Key header.

        Args:
          strategy: Strategy configuration for instantiating a strategy

          effective_date: Filter by date

          effective_time: Filter by time

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/v1/strategy/signals/entry",
            body=await async_maybe_transform(
                {
                    "strategy": strategy,
                    "effective_date": effective_date,
                    "effective_time": effective_time,
                },
                signal_get_entry_signals_params.SignalGetEntrySignalsParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SignalGetEntrySignalsResponse,
        )

    async def get_exit_signals(
        self,
        *,
        portfolio: Iterable[signal_get_exit_signals_params.Portfolio],
        strategy: StrategyConfigParam,
        effective_date: Union[str, date, None] | Omit = omit,
        effective_time: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SignalGetExitSignalsResponse:
        """
        Get exit signals for the given portfolio positions

        Requires authentication via X-API-Key header.

        Args:
          strategy: Strategy configuration for instantiating a strategy

          effective_date: Filter by date

          effective_time: Filter by time

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/v1/strategy/signals/exit",
            body=await async_maybe_transform(
                {
                    "portfolio": portfolio,
                    "strategy": strategy,
                    "effective_date": effective_date,
                    "effective_time": effective_time,
                },
                signal_get_exit_signals_params.SignalGetExitSignalsParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SignalGetExitSignalsResponse,
        )


class SignalsResourceWithRawResponse:
    def __init__(self, signals: SignalsResource) -> None:
        self._signals = signals

        self.get_entry_signals = to_raw_response_wrapper(
            signals.get_entry_signals,
        )
        self.get_exit_signals = to_raw_response_wrapper(
            signals.get_exit_signals,
        )


class AsyncSignalsResourceWithRawResponse:
    def __init__(self, signals: AsyncSignalsResource) -> None:
        self._signals = signals

        self.get_entry_signals = async_to_raw_response_wrapper(
            signals.get_entry_signals,
        )
        self.get_exit_signals = async_to_raw_response_wrapper(
            signals.get_exit_signals,
        )


class SignalsResourceWithStreamingResponse:
    def __init__(self, signals: SignalsResource) -> None:
        self._signals = signals

        self.get_entry_signals = to_streamed_response_wrapper(
            signals.get_entry_signals,
        )
        self.get_exit_signals = to_streamed_response_wrapper(
            signals.get_exit_signals,
        )


class AsyncSignalsResourceWithStreamingResponse:
    def __init__(self, signals: AsyncSignalsResource) -> None:
        self._signals = signals

        self.get_entry_signals = async_to_streamed_response_wrapper(
            signals.get_entry_signals,
        )
        self.get_exit_signals = async_to_streamed_response_wrapper(
            signals.get_exit_signals,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List
from datetime import date

from .._models import BaseModel

__all__ = ["StrategyGetTradingDaysResponse"]


class StrategyGetTradingDaysResponse(BaseModel):
    trading_days: List[date]
